@@ -1,6 +1,7 @@
 /********************************************
 cast.c
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: cast.c,v 1.9 2009/09/16 09:29:51 tom Exp $
+ * $MawkId: cast.c,v 1.16 2012/06/27 21:54:40 tom Exp $
  * @Log: cast.c,v @
  * Revision 1.6  1996/08/11 22:07:50  mike
  * Fix small bozo in rt_error("overflow converting ...")
@@ -146,7 +147,8 @@ cast2_to_d(CELL * cp)
     }
     cp->type = C_DOUBLE;
 
-  two:cp++;
+  two:
+    cp++;
 
     switch (cp->type) {
     case C_NOINIT:
@@ -293,6 +295,7 @@ cast_to_RE(CELL * cp)
 	cast1_to_s(cp);
 
     p = re_compile(string(cp));
+    no_leaks_re_ptr(p);
     free_STRING(string(cp));
     cp->type = C_RE;
     cp->ptr = p;
@@ -305,7 +308,7 @@ cast_for_split(CELL * cp)
     static char meta[] = "^$.*+?|[]()";
     static char xbuff[] = "\\X";
     int c;
-    unsigned len;
+    size_t len;
 
     if (cp->type < C_STRING)
 	cast1_to_s(cp);
@@ -384,15 +387,21 @@ check_strnum(CELL * cp)
 	cp->dval = strtod((char *) s, &temp);
 	/* make overflow pure string */
 	if (errno && cp->dval != 0.0)
-	    return;
+	    break;
 #else
 	cp->dval = strtod((char *) s, &temp);
 #endif
+#ifdef ERANGE
+	if (errno == ERANGE)
+	    break;
+#endif
 
-	if ((char *) q <= temp)
+	if ((char *) q <= temp) {
 	    cp->type = C_STRNUM;
-	/*  <= instead of == , for some buggy strtod
-	   e.g. Apple Unix */
+	    /*  <= instead of == , for some buggy strtod
+	       e.g. Apple Unix */
+	}
+	break;
     }
 }
 
@@ -442,3 +451,52 @@ d_to_U(double d)
 	return (UInt) d;
     return 0;
 }
+
+#ifdef NO_LEAKS
+typedef struct _all_cells {
+    struct _all_cells *next;
+    char ptr;
+    CELL *cp;
+} ALL_CELLS;
+
+static ALL_CELLS *all_cells;
+/*
+ * Some regular expressions are parsed, and the pointer stored in the byte-code
+ * where we cannot distinguish it from other constants.  Keep a list here, to
+ * free on exit for auditing.
+ */
+void
+no_leaks_cell(CELL * cp)
+{
+    ALL_CELLS *p = calloc(1, sizeof(ALL_CELLS));
+    p->next = all_cells;
+    p->cp = cp;
+    p->ptr = 0;
+    all_cells = p;
+}
+
+void
+no_leaks_cell_ptr(CELL * cp)
+{
+    ALL_CELLS *p = calloc(1, sizeof(ALL_CELLS));
+    p->next = all_cells;
+    p->cp = cp;
+    p->ptr = 1;
+    all_cells = p;
+}
+
+void
+cell_leaks(void)
+{
+    while (all_cells != 0) {
+	ALL_CELLS *next = all_cells->next;
+	if (all_cells->ptr) {
+	    zfree(all_cells->cp, sizeof(CELL));
+	} else {
+	    free_cell_data(all_cells->cp);
+	}
+	free(all_cells);
+	all_cells = next;
+    }
+}
+#endif
