@@ -1,6 +1,7 @@
 /********************************************
 execute.c
-copyright 1991-1996, Michael D. Brennan
+copyright 2008-2010,2012, Thomas E. Dickey
+copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: execute.c,v 1.11 2009/12/17 10:33:20 tom Exp $
+ * $MawkId: execute.c,v 1.26 2012/06/27 17:15:52 tom Exp $
  * @Log: execute.c,v @
  * Revision 1.13  1996/02/01  04:39:40  mike
  * dynamic array scheme
@@ -91,6 +92,7 @@ the GNU General Public License, version 2, 1991.
 */
 
 #include "mawk.h"
+#include "files.h"
 #include "code.h"
 #include "memory.h"
 #include "symtype.h"
@@ -200,6 +202,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 #ifdef	DEBUG
     CELL *entry_sp = sp;
 #endif
+    int force_exit = (end_start == 0);
 
     if (fp) {
 	/* we are a function call, check for deep recursion */
@@ -218,8 +221,8 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    old_stack_base = (CELL *) 0;
     }
 
-    while (1)
-	switch (cdp++->op) {
+    while (1) {
+	switch ((cdp++)->op) {
 
 /* HALT only used by the disassemble now ; this remains
    so compilers don't offset the jump table */
@@ -234,19 +237,19 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	case _PUSHC:
 	    inc_sp();
-	    cellcpy(sp, cdp++->ptr);
+	    cellcpy(sp, (cdp++)->ptr);
 	    break;
 
 	case _PUSHD:
 	    inc_sp();
 	    sp->type = C_DOUBLE;
-	    sp->dval = *(double *) cdp++->ptr;
+	    sp->dval = *(double *) (cdp++)->ptr;
 	    break;
 
 	case _PUSHS:
 	    inc_sp();
 	    sp->type = C_STRING;
-	    sp->ptr = cdp++->ptr;
+	    sp->ptr = (cdp++)->ptr;
 	    string(sp)->ref_cnt++;
 	    break;
 
@@ -261,14 +264,13 @@ execute(INST * cdp,		/* code ptr, start execution here */
 			 SAMESEG(cp, field) &&
 #endif
 			 cp >= NF && cp <= LAST_PFIELD)) {
-		    /* its a real field $1, $2 ...
-		       If its greater than $NF, we have to
-		       make sure its set to ""  so that
+		    /* it is a real field $1, $2 ...
+		       If it is greater than $NF, we have to
+		       make sure it is set to ""  so that
 		       (++|--) and g?sub() work right
 		     */
 		    t = field_addr_to_index(cp);
 		    if (t > nf) {
-			cell_destroy(cp);
 			cp->type = C_STRING;
 			cp->ptr = (PTR) & null_str;
 			null_str.ref_cnt++;
@@ -280,26 +282,26 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	case _PUSHA:
 	case A_PUSHA:
 	    inc_sp();
-	    sp->ptr = cdp++->ptr;
+	    sp->ptr = (cdp++)->ptr;
 	    break;
 
 	case _PUSHI:
 	    /* put contents of next address on stack */
 	    inc_sp();
-	    cellcpy(sp, cdp++->ptr);
+	    cellcpy(sp, (cdp++)->ptr);
 	    break;
 
 	case L_PUSHI:
 	    /* put the contents of a local var on stack,
 	       cdp->op holds the offset from the frame pointer */
 	    inc_sp();
-	    cellcpy(sp, fp + cdp++->op);
+	    cellcpy(sp, fp + (cdp++)->op);
 	    break;
 
 	case L_PUSHA:
 	    /* put a local address on eval stack */
 	    inc_sp();
-	    sp->ptr = (PTR) (fp + cdp++->op);
+	    sp->ptr = (PTR) (fp + (cdp++)->op);
 	    break;
 
 	case F_PUSHI:
@@ -373,7 +375,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       array, replace the expr with the cell address inside
 	       the array */
 
-	    cp = array_find((ARRAY) cdp++->ptr, sp, CREATE);
+	    cp = array_find((ARRAY) (cdp++)->ptr, sp, CREATE);
 	    cell_destroy(sp);
 	    sp->ptr = (PTR) cp;
 	    break;
@@ -383,7 +385,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       array, replace the expr with the contents of the
 	       cell inside the array */
 
-	    cp = array_find((ARRAY) cdp++->ptr, sp, CREATE);
+	    cp = array_find((ARRAY) (cdp++)->ptr, sp, CREATE);
 	    cell_destroy(sp);
 	    cellcpy(sp, cp);
 	    break;
@@ -394,9 +396,11 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field, replace expr
 	       with  array[expr]
 	     */
-	    cp = array_find((ARRAY) fp[cdp++->op].ptr, sp, CREATE);
-	    cell_destroy(sp);
-	    cellcpy(sp, cp);
+	    if (fp != 0) {
+		cp = array_find((ARRAY) fp[(cdp++)->op].ptr, sp, CREATE);
+		cell_destroy(sp);
+		cellcpy(sp, cp);
+	    }
 	    break;
 
 	case LAE_PUSHA:
@@ -405,9 +409,11 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field, replace expr
 	       with  & array[expr]
 	     */
-	    cp = array_find((ARRAY) fp[cdp++->op].ptr, sp, CREATE);
-	    cell_destroy(sp);
-	    sp->ptr = (PTR) cp;
+	    if (fp != 0) {
+		cp = array_find((ARRAY) fp[(cdp++)->op].ptr, sp, CREATE);
+		cell_destroy(sp);
+		sp->ptr = (PTR) cp;
+	    }
 	    break;
 
 	case LA_PUSHA:
@@ -415,18 +421,19 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field. Push this ARRAY
 	       on the eval stack
 	     */
-	    inc_sp();
-	    sp->ptr = fp[cdp++->op].ptr;
+	    if (fp != 0) {
+		inc_sp();
+		sp->ptr = fp[(cdp++)->op].ptr;
+	    }
 	    break;
 
 	case SET_ALOOP:
 	    {
 		ALOOP_STATE *ap = ZMALLOC(ALOOP_STATE);
-		unsigned vector_size;
+		size_t vector_size;
 
 		ap->var = (CELL *) sp[-1].ptr;
-		ap->base = ap->ptr = array_loop_vector(
-							  (ARRAY) sp->ptr, &vector_size);
+		ap->base = ap->ptr = array_loop_vector((ARRAY) sp->ptr, &vector_size);
 		ap->limit = ap->base + vector_size;
 		sp -= 2;
 
@@ -440,13 +447,14 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	case ALOOP:
 	    {
 		ALOOP_STATE *ap = aloop_state;
-		if (ap->ptr < ap->limit) {
+		if (ap != 0 && (ap->ptr < ap->limit)) {
 		    cell_destroy(ap->var);
 		    ap->var->type = C_STRING;
 		    ap->var->ptr = (PTR) * ap->ptr++;
 		    cdp += cdp->op;
-		} else
+		} else {
 		    cdp++;
+		}
 	    }
 	    break;
 
@@ -454,17 +462,19 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    {
 		/* finish up an array loop */
 		ALOOP_STATE *ap = aloop_state;
-		aloop_state = ap->link;
-		while (ap->ptr < ap->limit) {
-		    free_STRING(*ap->ptr);
-		    ap->ptr++;
+		if (ap != 0) {
+		    aloop_state = ap->link;
+		    while (ap->ptr < ap->limit) {
+			free_STRING(*ap->ptr);
+			ap->ptr++;
+		    }
+		    if (ap->base < ap->limit) {
+			zfree(ap->base,
+			      ((unsigned) (ap->limit - ap->base)
+			       * sizeof(STRING *)));
+		    }
+		    ZFREE(ap);
 		}
-		if (ap->base < ap->limit) {
-		    zfree(ap->base,
-			  ((unsigned) (ap->limit - ap->base)
-			   * sizeof(STRING *)));
-		}
-		ZFREE(ap);
 	    }
 	    break;
 
@@ -843,7 +853,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	case _CAT:
 	    {
-		unsigned len1, len2;
+		size_t len1, len2;
 		char *str1, *str2;
 		STRING *b;
 
@@ -867,12 +877,12 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	case _PUSHINT:
 	    inc_sp();
-	    sp->type = cdp++->op;
+	    sp->type = (short) (cdp++)->op;
 	    break;
 
 	case _BUILTIN:
 	case _PRINT:
-	    sp = (*(PF_CP) cdp++->ptr) (sp);
+	    sp = (*(PF_CP) (cdp++)->ptr) (sp);
 	    break;
 
 	case _POST_INC:
@@ -1039,7 +1049,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		sp->type = C_DOUBLE;
 		sp->dval = (REtest(string(field)->str,
 				   string(field)->len,
-				   cast_to_re(cdp++->ptr))
+				   cast_to_re((cdp++)->ptr))
 			    ? 1.0
 			    : 0.0);
 
@@ -1055,7 +1065,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		cast1_to_s(sp);
 	    t = REtest(string(sp)->str,
 		       string(sp)->len,
-		       cast_to_re(cdp++->ptr));
+		       cast_to_re((cdp++)->ptr));
 	    free_STRING(string(sp));
 	    sp->type = C_DOUBLE;
 	    sp->dval = t ? 1.0 : 0.0;
@@ -1072,6 +1082,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		       cast_to_re((sp + 1)->ptr));
 
 	    free_STRING(string(sp));
+	    no_leaks_re_ptr((sp + 1)->ptr);
 	    sp->type = C_DOUBLE;
 	    sp->dval = t ? 1.0 : 0.0;
 	    break;
@@ -1106,7 +1117,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	    /* form a multiple array index */
 	case A_CAT:
-	    sp = array_cat(sp, cdp++->op);
+	    sp = array_cat(sp, (cdp++)->op);
 	    break;
 
 	case _EXIT:
@@ -1118,23 +1129,30 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	case _EXIT0:
 
-	    if (!end_start)
+	    if (force_exit)
 		mawk_exit(exit_code);
 
 	    cdp = end_start;
-	    end_start = (INST *) 0;	/* makes sure next exit exits */
+	    force_exit = 1;	/* makes sure next exit exits */
 
-	    if (begin_start)
-		zfree(begin_start, begin_size);
-	    if (main_start)
-		zfree(main_start, main_size);
+	    if (begin_start) {
+		free_codes("BEGIN", begin_start, begin_size);
+		begin_start = 0;
+		begin_size = 0;
+	    }
+	    if (main_start) {
+		free_codes("MAIN", main_start, main_size);
+		main_start = 0;
+		main_size = 0;
+	    }
 	    sp = eval_stack - 1;	/* might be in user function */
 	    CLEAR_ALOOP_STACK();	/* ditto */
 	    break;
 
 	case _JMAIN:		/* go from BEGIN code to MAIN code */
-	    zfree(begin_start, begin_size);
-	    begin_start = (INST *) 0;
+	    free_codes("BEGIN", begin_start, begin_size);
+	    begin_start = 0;
+	    begin_size = 0;
 	    cdp = main_start;
 	    break;
 
@@ -1151,18 +1169,26 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    cdp = next_label;
 	    break;
 
+	case _NEXTFILE:
+	    /* nextfile might be inside an aloop -- clear stack */
+	    CLEAR_ALOOP_STACK();
+	    FINsemi_close(main_fin);
+	    cdp = next_label;
+	    break;
+
 	case OL_GL:
 	    {
 		char *p;
-		unsigned len;
+		size_t len;
 
 		if (!(p = FINgets(main_fin, &len))) {
-		    if (!end_start)
+		    if (force_exit)
 			mawk_exit(0);
 
 		    cdp = end_start;
 		    zfree(main_start, main_size);
-		    main_start = end_start = (INST *) 0;
+		    main_start = (INST *) 0;
+		    force_exit = 1;
 		} else {
 		    set_field0(p, len);
 		    cdp = restart_label;
@@ -1177,15 +1203,16 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	case OL_GL_NR:
 	    {
 		char *p;
-		unsigned len;
+		size_t len;
 
 		if (!(p = FINgets(main_fin, &len))) {
-		    if (!end_start)
+		    if (force_exit)
 			mawk_exit(0);
 
 		    cdp = end_start;
 		    zfree(main_start, main_size);
-		    main_start = end_start = (INST *) 0;
+		    main_start = (INST *) 0;
+		    force_exit = 1;
 		} else {
 		    set_field0(p, len);
 		    cdp = restart_label;
@@ -1211,13 +1238,13 @@ execute(INST * cdp,		/* code ptr, start execution here */
        cdp[4] -- start of pat1 code
 */
 
-#define FLAG	cdp[0].op
-#define PAT2	cdp[1].op
-#define ACTION	  cdp[2].op
-#define FOLLOW	  cdp[3].op
-#define PAT1	  4
+#define FLAG      cdp[0].op
+#define PAT2      cdp[1].op
+#define ACTION    cdp[2].op
+#define FOLLOW    cdp[3].op
+#define PAT1      4
 
-	    if (FLAG)		/* test again pat1 */
+	    if (FLAG)		/* test against pat1 */
 	    {
 		execute(cdp + PAT1, sp, fp);
 		t = test(sp + 1);
@@ -1272,8 +1299,8 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	     */
 
 	    {
-		FBLOCK *fbp = (FBLOCK *) cdp++->ptr;
-		int a_args = cdp++->op;		/* actual number of args */
+		FBLOCK *fbp = (FBLOCK *) (cdp++)->ptr;
+		int a_args = (cdp++)->op;	/* actual number of args */
 		CELL *nfp = sp - a_args + 1;	/* new fp for callee */
 		CELL *local_p = sp + 1;		/* first local argument on stack */
 		char *type_p = 0;	/* pts to type of an argument */
@@ -1296,7 +1323,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 		/* cleanup the callee's arguments */
 		/* putting return value at top of eval stack */
-		if (sp >= nfp) {
+		if ((type_p != 0) && (sp >= nfp)) {
 		    cp = sp + 1;	/* cp -> the function return */
 
 		    do {
@@ -1305,8 +1332,9 @@ execute(INST * cdp,		/* code ptr, start execution here */
 				array_clear(sp->ptr);
 				ZFREE((ARRAY) sp->ptr);
 			    }
-			} else
+			} else {
 			    cell_destroy(sp);
+			}
 
 			type_p--;
 			sp--;
@@ -1324,6 +1352,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	default:
 	    bozo("bad opcode");
 	}
+    }
 }
 
 /*
@@ -1358,26 +1387,31 @@ test(CELL * cp)
 static int
 compare(CELL * cp)
 {
-    int k;
+    int result;
 
   reswitch:
+    result = 0;
 
     switch (TEST2(cp)) {
     case TWO_NOINITS:
-	return 0;
+	break;
 
     case TWO_DOUBLES:
       two_d:
-	return cp->dval > (cp + 1)->dval ? 1 :
-	    cp->dval < (cp + 1)->dval ? -1 : 0;
+	result = ((cp->dval > (cp + 1)->dval)
+		  ? 1
+		  : ((cp->dval < (cp + 1)->dval)
+		     ? -1
+		     : 0));
+	break;
 
     case TWO_STRINGS:
     case STRING_AND_STRNUM:
       two_s:
-	k = strcmp(string(cp)->str, string(cp + 1)->str);
+	result = strcmp(string(cp)->str, string(cp + 1)->str);
 	free_STRING(string(cp));
 	free_STRING(string(cp + 1));
-	return k;
+	break;
 
     case NOINIT_AND_DOUBLE:
     case NOINIT_AND_STRNUM:
@@ -1403,8 +1437,9 @@ compare(CELL * cp)
 
     default:			/* there are no default cases */
 	bozo("bad cell type passed to compare");
+	break;
     }
-    return 0;			/* shut up */
+    return result;
 }
 
 /* does not assume target was a cell, if so
@@ -1462,8 +1497,7 @@ DB_cell_destroy(CELL * cp)
     case C_MBSTRN:
     case C_STRING:
     case C_STRNUM:
-	if (--string(cp)->ref_cnt == 0)
-	    zfree(string(cp), string(cp)->len + STRING_OH);
+	free_STRING(string(cp));
 	break;
 
     case C_RE:
