@@ -1,6 +1,7 @@
 /********************************************
 fcall.c
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 1991-1993,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fcall.c,v 1.6 2009/12/13 19:36:24 Jonathan.Nieder Exp $
+ * $MawkId: fcall.c,v 1.8 2010/12/10 17:00:00 tom Exp $
  * @Log: fcall.c,v @
  * Revision 1.7  1995/08/27  15:46:47  mike
  * change some errmsgs to compile_errors
@@ -293,8 +294,8 @@ check_fcall(
 	/* forward reference to a function to be defined later */
 	p = ZMALLOC(FCALL_REC);
 	p->callee = callee;
-	p->call_scope = call_scope;
-	p->move_level = move_level;
+	p->call_scope = (short) call_scope;
+	p->move_level = (short) move_level;
 	p->call = call;
 	p->arg_list = arg_list;
 	p->arg_cnt_checked = 0;
@@ -309,8 +310,8 @@ check_fcall(
 				       code_base))) {
 	    p = ZMALLOC(FCALL_REC);
 	    p->callee = callee;
-	    p->call_scope = call_scope;
-	    p->move_level = move_level;
+	    p->call_scope = (short) call_scope;
+	    p->move_level = (short) move_level;
 	    p->call = call;
 	    p->arg_list = arg_list;
 	    p->arg_cnt_checked = 1;
@@ -367,7 +368,8 @@ relocate_arglist(
 
     /* Note: call_offset is always set even for args that don't need to
        be patched so that this check works. */
-    if (p->call_offset < offset || (unsigned) p->call_offset >= offset + len)
+    if (p->call_offset < offset ||
+	(unsigned) p->call_offset >= (unsigned) offset + len)
 	return;
 
     /* relocate the whole list */
