@@ -1,6 +1,7 @@
 /********************************************
 files.c
-copyright 1991-94.  Michael D. Brennan
+copyright 2008-2010,2012.  Thomas E. Dickey
+copyright 1991-1994,1996.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,8 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: files.c,v 1.12 2009/12/17 00:06:51 tom Exp $
+ * $MawkId: files.c,v 1.21 2012/06/27 17:57:36 tom Exp $
+ *
  * @Log: files.c,v @
  * Revision 1.9  1996/01/14  17:14:10  mike
  * flush_all_output()
@@ -52,8 +54,7 @@ the GNU General Public License, version 2, 1991.
  *
  * Revision 5.1	 91/12/05  07:56:00  brennan
  * 1.1 pre-release
- *
-*/
+ */
 
 /* files.c */
 
@@ -64,7 +65,6 @@ the GNU General Public License, version 2, 1991.
 #include "init.h"
 
 #include <sys/types.h>
-#include <unistd.h>
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -108,11 +108,36 @@ static FILE *tfopen(const char *, const char *);
 static void efflush(FILE *);
 static void close_error(FILE_NODE * p);
 
+static FILE_NODE *
+alloc_filenode(void)
+{
+    FILE_NODE *result;
+
+    result = ZMALLOC(FILE_NODE);
+
+#ifdef NO_LEAKS
+    result->name = 0;
+#endif
+
+    return result;
+}
+
+static void
+free_filenode(FILE_NODE * p)
+{
+#ifdef NO_LEAKS
+    if (p->name != 0) {
+	free_STRING(p->name);
+    }
+#endif
+    zfree(p, sizeof(FILE_NODE));
+}
+
 /* find a file on file_list */
 PTR
 file_find(STRING * sval, int type)
 {
-    register FILE_NODE *p = file_list;
+    FILE_NODE *p = file_list;
     FILE_NODE *q = (FILE_NODE *) 0;
     char *name = sval->str;
     const char *ostr;
@@ -120,7 +145,7 @@ file_find(STRING * sval, int type)
     while (1) {
 	if (!p) {
 	    /* open a new one */
-	    p = ZMALLOC(FILE_NODE);
+	    p = alloc_filenode();
 
 	    switch (p->type = (short) type) {
 	    case F_TRUNC:
@@ -145,7 +170,7 @@ file_find(STRING * sval, int type)
 
 	    case F_IN:
 		if (!(p->ptr = (PTR) FINopen(name, 0))) {
-		    zfree(p, sizeof(FILE_NODE));
+		    free_filenode(p);
 		    return (PTR) 0;
 		}
 		break;
@@ -159,7 +184,7 @@ file_find(STRING * sval, int type)
 		    if (type == PIPE_OUT)
 			goto out_failure;
 		    else {
-			zfree(p, sizeof(FILE_NODE));
+			free_filenode(p);
 			return (PTR) 0;
 		    }
 		}
@@ -203,10 +228,10 @@ file_find(STRING * sval, int type)
   out_failure:
     errmsg(errno, "cannot open \"%s\" for output", name);
     mawk_exit(2);
-
+    /* NOTREACHED */
 }
 
-/* Close a file and delete it's node from the file_list.
+/* Close a file and delete its node from the file_list.
    Walk the whole list, in case a name has two nodes,
    e.g. < "/dev/tty" and > "/dev/tty"
 */
@@ -215,7 +240,7 @@ int
 file_close(STRING * sval)
 {
     FILE_NODE dummy;
-    register FILE_NODE *p;
+    FILE_NODE *p;
     FILE_NODE *q = &dummy;	/* trails p */
     FILE_NODE *hold;
     char *name = sval->str;
@@ -238,15 +263,16 @@ file_close(STRING * sval)
 	    switch (p->type) {
 	    case F_TRUNC:
 	    case F_APPEND:
-		if (fclose((FILE *) p->ptr) != 0)
+		if (fclose((FILE *) p->ptr) != 0) {
 		    close_error(p);
+		}
 		retval = 0;
 		break;
 
 	    case PIPE_OUT:
-		if (fclose((FILE *) p->ptr) != 0)
+		if (fclose((FILE *) p->ptr) != 0) {
 		    close_error(p);
-
+		}
 #ifdef  HAVE_REAL_PIPES
 		retval = wait_for(p->pid);
 #endif
@@ -276,10 +302,9 @@ file_close(STRING * sval)
 		break;
 	    }
 
-	    free_STRING(p->name);
 	    hold = p;
 	    p = p->link;
-	    ZFREE(hold);
+	    free_filenode(hold);
 	} else {
 	    q = p;
 	    p = p->link;
@@ -297,8 +322,8 @@ int
 file_flush(STRING * sval)
 {
     int ret = -1;
-    register FILE_NODE *p = file_list;
-    unsigned len = sval->len;
+    FILE_NODE *p = file_list;
+    size_t len = sval->len;
     char *str = sval->str;
 
     if (len == 0) {
@@ -353,21 +378,27 @@ efflush(FILE *fp)
 void
 close_out_pipes(void)
 {
-    register FILE_NODE *p = file_list;
+    FILE_NODE *p = file_list;
+    FILE_NODE *q = 0;
 
     while (p) {
+
 	if (IS_OUTPUT(p->type)) {
 	    if (fclose((FILE *) p->ptr) != 0) {
 		/* if another error occurs we do not want to be called
 		   for the same file again */
 
-		file_list = p->link;
+		if (q != 0)
+		    q->link = p->link;
+		else
+		    file_list = p->link;
 		close_error(p);
-	    }
-	    if (p->type == PIPE_OUT)
+	    } else if (p->type == PIPE_OUT) {
 		wait_for(p->pid);
+	    }
 	}
 
+	q = p;
 	p = p->link;
     }
 }
@@ -378,7 +409,7 @@ close_out_pipes(void)
 void
 close_fake_pipes(void)
 {
-    register FILE_NODE *p = file_list;
+    FILE_NODE *p = file_list;
     char xbuff[100];
 
     /* close input pipes first to free descriptors for children */
@@ -393,8 +424,9 @@ close_fake_pipes(void)
     p = file_list;
     while (p) {
 	if (p->type == PIPE_OUT) {
-	    if (fclose(p->ptr) != 0)
+	    if (fclose(p->ptr) != 0) {
 		close_error(p);
+	    }
 	    close_fake_outpipe(p->name->str, p->pid);
 	}
 	p = p->link;
@@ -445,8 +477,9 @@ get_pipe(char *name, int type, int *pid_ptr)
 	break;
     }
 
-    return type == PIPE_IN ? (PTR) FINdopen(local_fd, 0) :
-	(PTR) fdopen(local_fd, "w");
+    return ((type == PIPE_IN)
+	    ? (PTR) FINdopen(local_fd, 0)
+	    : (PTR) fdopen(local_fd, "w"));
 }
 
 /*------------ children ------------------*/
@@ -465,7 +498,7 @@ static struct child {
 static void
 add_to_child_list(int pid, int exit_status)
 {
-    register struct child *p = ZMALLOC(struct child);
+    struct child *p = ZMALLOC(struct child);
 
     p->pid = pid;
     p->exit_status = exit_status;
@@ -477,7 +510,7 @@ static struct child *
 remove_from_child_list(int pid)
 {
     struct child dummy;
-    register struct child *p;
+    struct child *p;
     struct child *q = &dummy;
 
     dummy.link = p = child_list;
@@ -506,7 +539,7 @@ remove_from_child_list(int pid)
 int
 wait_for(int pid)
 {
-    int exit_status;
+    int exit_status = 0;
     struct child *p;
     int id;
 
@@ -541,28 +574,37 @@ wait_for(int pid)
 
 #endif /* HAVE_REAL_PIPES */
 
+/*
+ * Create names by which the standard streams can be referenced in a script.
+ */
 void
-set_stderr(void)		/* and stdout */
+set_stdio(void)
 {
-    FILE_NODE *p, *q;
+    FILE_NODE *p, *q, *r;
 
     /* We insert stderr first to get it at the end of the list. This is
        needed because we want to output errors encountered on closing
        stdout. */
 
-    q = ZMALLOC(FILE_NODE);
+    q = alloc_filenode();
     q->link = (FILE_NODE *) 0;
     q->type = F_TRUNC;
     q->name = new_STRING("/dev/stderr");
     q->ptr = (PTR) stderr;
 
-    p = ZMALLOC(FILE_NODE);
+    p = alloc_filenode();
     p->link = q;
     p->type = F_TRUNC;
     p->name = new_STRING("/dev/stdout");
     p->ptr = (PTR) stdout;
 
-    file_list = p;
+    r = alloc_filenode();
+    r->link = p;
+    r->type = F_IN;
+    r->name = new_STRING("/dev/stdin");
+    r->ptr = (PTR) stdin;
+
+    file_list = r;
 }
 
 /* fopen() but no buffering to ttys */
@@ -615,6 +657,23 @@ stdout_init(void)
 static void
 close_error(FILE_NODE * p)
 {
+    TRACE(("close_error(%s)\n", p->name->str));
     errmsg(errno, "close failed on file %s", p->name->str);
+#ifdef NO_LEAKS
+    free_filenode(p);
+#endif
     mawk_exit(2);
 }
+
+#ifdef NO_LEAKS
+void
+files_leaks(void)
+{
+    TRACE(("files_leaks\n"));
+    while (file_list != 0) {
+	FILE_NODE *p = file_list;
+	file_list = p->link;
+	free_filenode(p);
+    }
+}
+#endif
