@@ -1,6 +1,7 @@
 /********************************************
 fin.c
-copyright 1991, 1992.  Michael D. Brennan
+copyright 2008-2010,2012.  Thomas E. Dickey
+copyright 1991-1995,1996.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fin.c,v 1.22 2009/12/17 00:06:57 tom Exp $
+ * $MawkId: fin.c,v 1.34 2012/06/27 12:14:13 tom Exp $
  * @Log: fin.c,v @
  * Revision 1.10  1995/12/24  22:23:22  mike
  * remove errmsg() from inside FINopen
@@ -91,32 +92,49 @@ static FIN *next_main(int);
 static char *enlarge_fin_buffer(FIN *);
 int is_cmdline_assign(char *);	/* also used by init */
 
+/* this is how we mark EOF on main_fin  */
+static char dead_buff = 0;
+static FIN dead_main =
+{0, (FILE *) 0, &dead_buff, &dead_buff, &dead_buff,
+ 1, EOF_FLAG};
+
+static void
+free_fin_data(FIN * fin)
+{
+    if (fin != &dead_main) {
+	zfree(fin->buff, (size_t) (fin->nbuffs * BUFFSZ + 1));
+	ZFREE(fin);
+    }
+}
+
 /* convert file-descriptor to FIN*.
    It's the main stream if main_flag is set
 */
 FIN *
 FINdopen(int fd, int main_flag)
 {
-    register FIN *fin = ZMALLOC(FIN);
+    FIN *fin = ZMALLOC(FIN);
 
     fin->fd = fd;
     fin->flags = main_flag ? (MAIN_FLAG | START_FLAG) : START_FLAG;
-    fin->buffp = fin->buff = (char *) zmalloc(BUFFSZ + 1);
+    fin->buffp = fin->buff = (char *) zmalloc((size_t) BUFFSZ + 1);
     fin->limit = fin->buffp;
     fin->nbuffs = 1;
     fin->buff[0] = 0;
 
     if ((isatty(fd) && rs_shadow.type == SEP_CHAR && rs_shadow.c == '\n')
-	|| (interactive_flag && fd == 0)) {
+	|| interactive_flag) {
 	/* interactive, i.e., line buffer this file */
-	if (fd == 0)
+	if (fd == 0) {
 	    fin->fp = stdin;
-	else if (!(fin->fp = fdopen(fd, "r"))) {
+	} else if (!(fin->fp = fdopen(fd, "r"))) {
 	    errmsg(errno, "fdopen failed");
+	    free_fin_data(fin);
 	    mawk_exit(2);
 	}
-    } else
+    } else {
 	fin->fp = (FILE *) 0;
+    }
 
     return fin;
 }
@@ -129,6 +147,7 @@ FINdopen(int fd, int main_flag)
 FIN *
 FINopen(char *filename, int main_flag)
 {
+    FIN *result = 0;
     int fd;
     int oflag = O_RDONLY;
 
@@ -143,13 +162,11 @@ FINopen(char *filename, int main_flag)
 	if (bm)
 	    setmode(0, O_BINARY);
 #endif
-	return FINdopen(0, main_flag);
+	result = FINdopen(0, main_flag);
+    } else if ((fd = open(filename, oflag, 0)) != -1) {
+	result = FINdopen(fd, main_flag);
     }
-
-    if ((fd = open(filename, oflag, 0)) == -1)
-	return (FIN *) 0;
-    else
-	return FINdopen(fd, main_flag);
+    return result;
 }
 
 /* frees the buffer and fd, but leaves FIN structure until
@@ -161,7 +178,7 @@ FINsemi_close(FIN * fin)
     static char dead = 0;
 
     if (fin->buff != &dead) {
-	zfree(fin->buff, fin->nbuffs * BUFFSZ + 1);
+	zfree(fin->buff, (size_t) (fin->nbuffs * BUFFSZ + 1));
 
 	if (fin->fd) {
 	    if (fin->fp)
@@ -190,12 +207,12 @@ FINclose(FIN * fin)
 */
 
 char *
-FINgets(FIN * fin, unsigned *len_p)
+FINgets(FIN * fin, size_t *len_p)
 {
     char *p;
     char *q = 0;
-    unsigned match_len;
-    unsigned r;
+    size_t match_len;
+    size_t r;
 
   restart:
 
@@ -232,7 +249,7 @@ FINgets(FIN * fin, unsigned *len_p)
 	    }
 	} else {
 	    /* block buffering */
-	    r = fillbuff(fin->fd, fin->buff, fin->nbuffs * BUFFSZ);
+	    r = fillbuff(fin->fd, fin->buff, (size_t) (fin->nbuffs * BUFFSZ));
 	    if (r == 0) {
 		fin->flags |= EOF_FLAG;
 		fin->buffp = fin->buff;
@@ -269,14 +286,14 @@ FINgets(FIN * fin, unsigned *len_p)
 
     case SEP_STR:
 	q = str_str(p,
-		    (unsigned) (fin->limit - p),
+		    (size_t) (fin->limit - p),
 		    ((STRING *) rs_shadow.ptr)->str,
 		    match_len = ((STRING *) rs_shadow.ptr)->len);
 	break;
 
     case SEP_MLR:
     case SEP_RE:
-	q = re_pos_match(p, (unsigned) (fin->limit - p), rs_shadow.ptr, &match_len);
+	q = re_pos_match(p, (size_t) (fin->limit - p), rs_shadow.ptr, &match_len);
 	/* if the match is at the end, there might still be
 	   more to match in the file */
 	if (q && q[match_len] == 0 && !(fin->flags & EOF_FLAG))
@@ -314,10 +331,10 @@ FINgets(FIN * fin, unsigned *len_p)
 	p = enlarge_fin_buffer(fin);
     } else {
 	/* move a partial line to front of buffer and try again */
-	unsigned rr;
-	unsigned amount = (unsigned) (fin->limit - p);
+	size_t rr;
+	size_t amount = (size_t) (fin->limit - p);
 
-	p = (char *) memcpy(fin->buff, p, r = (unsigned) (fin->limit - p));
+	p = (char *) memmove(fin->buff, p, r = (size_t) (fin->limit - p));
 	q = p + r;
 	rr = fin->nbuffs * BUFFSZ - r;
 
@@ -332,9 +349,9 @@ FINgets(FIN * fin, unsigned *len_p)
 static char *
 enlarge_fin_buffer(FIN * fin)
 {
-    unsigned r;
-    unsigned oldsize = fin->nbuffs * BUFFSZ + 1;
-    unsigned limit = (unsigned) (fin->limit - fin->buff);
+    size_t r;
+    size_t oldsize = fin->nbuffs * BUFFSZ + 1;
+    size_t limit = (size_t) (fin->limit - fin->buff);
 
 #ifdef  MSDOS
     /* I'm not sure this can really happen:
@@ -349,7 +366,7 @@ enlarge_fin_buffer(FIN * fin)
 	fin->buff = (char *) zrealloc(fin->buff, oldsize, oldsize + BUFFSZ);
     fin->nbuffs++;
 
-    r = fillbuff(fin->fd, fin->buff + (oldsize - 1), BUFFSZ);
+    r = fillbuff(fin->fd, fin->buff + (oldsize - 1), (size_t) BUFFSZ);
     if (r < BUFFSZ)
 	fin->flags |= EOF_FLAG;
 
@@ -361,14 +378,14 @@ enlarge_fin_buffer(FIN * fin)
   target is big enough to hold size + 1 chars
   on exit the back of the target is zero terminated
  *--------------*/
-unsigned
-fillbuff(int fd, char *target, unsigned size)
+size_t
+fillbuff(int fd, char *target, size_t size)
 {
     register int r;
-    unsigned entry_size = size;
+    size_t entry_size = size;
 
     while (size)
-	switch (r = read(fd, target, size)) {
+	switch (r = (int) read(fd, target, size)) {
 	case -1:
 	    errmsg(errno, "read error");
 	    mawk_exit(2);
@@ -384,7 +401,7 @@ fillbuff(int fd, char *target, unsigned size)
 
   out:
     *target = 0;
-    return entry_size - size;
+    return (size_t) (entry_size - size);
 }
 
 /* main_fin is a handle to the main input stream
@@ -442,15 +459,19 @@ static FIN *
 next_main(int open_flag)	/* called by open_main() if on */
 {
     register CELL *cp;
+    CELL *cp0;
     CELL argc;			/* copy of ARGC */
     CELL c_argi;		/* cell copy of argi */
     CELL argval;		/* copy of ARGV[c_argi] */
+    int failed = 1;
 
     argval.type = C_NOINIT;
     c_argi.type = C_DOUBLE;
 
-    if (main_fin)
+    if (main_fin) {
 	FINclose(main_fin);
+	main_fin = 0;
+    }
     /* FILENAME and FNR don't change unless we really open
        a new file */
 
@@ -462,21 +483,28 @@ next_main(int open_flag)	/* called by open_main() if on */
 	c_argi.dval = argi;
 	argi += 1.0;
 
-	if (!(cp = array_find(Argv, &c_argi, NO_CREATE)))
+	if (!(cp0 = array_find(Argv, &c_argi, NO_CREATE)))
 	    continue;		/* its deleted */
 
 	/* make a copy so we can cast w/o side effect */
 	cell_destroy(&argval);
-	cp = cellcpy(&argval, cp);
+	cp = cellcpy(&argval, cp0);
+#ifndef NO_LEAKS
+	cell_destroy(cp0);
+#endif
+
 	if (cp->type < C_STRING)
 	    cast1_to_s(cp);
-	if (string(cp)->len == 0)
+	if (string(cp)->len == 0) {
+	    /* file argument is "" */
+	    cell_destroy(cp);
 	    continue;
-	/* file argument is "" */
+	}
 
 	/* it might be a command line assignment */
-	if (is_cmdline_assign(string(cp)->str))
+	if (is_cmdline_assign(string(cp)->str)) {
 	    continue;
+	}
 
 	/* try to open it -- we used to continue on failure,
 	   but posix says we should quit */
@@ -488,34 +516,29 @@ next_main(int open_flag)	/* called by open_main() if on */
 	/* success -- set FILENAME and FNR */
 	cell_destroy(FILENAME);
 	cellcpy(FILENAME, cp);
-	free_STRING(string(cp));
+	cell_destroy(cp);
 	cell_destroy(FNR);
 	FNR->type = C_DOUBLE;
 	FNR->dval = 0.0;
 	rt_fnr = 0;
 
-	return main_fin;
-    }
-    /* failure */
-    cell_destroy(&argval);
-
-    if (open_flag) {
-	/* all arguments were null or assignment */
-	set_main_to_stdin();
-	return main_fin;
+	failed = 0;
+	break;
     }
 
-    /* real failure */
-    {
-	/* this is how we mark EOF on main_fin  */
-	static char dead_buff = 0;
-	static FIN dead_main =
-	{0, (FILE *) 0, &dead_buff, &dead_buff, &dead_buff,
-	 1, EOF_FLAG};
+    if (failed) {
+	cell_destroy(&argval);
 
-	return main_fin = &dead_main;
-	/* since MAIN_FLAG is not set, FINgets won't call next_main() */
+	if (open_flag) {
+	    /* all arguments were null or assignment */
+	    set_main_to_stdin();
+	} else {
+	    main_fin = &dead_main;
+	    /* since MAIN_FLAG is not set, FINgets won't call next_main() */
+	}
     }
+
+    return main_fin;
 }
 
 int
@@ -525,10 +548,10 @@ is_cmdline_assign(char *s)
     int c;
     SYMTAB *stp;
     CELL *cp = 0;
-    unsigned len;
+    size_t len;
     CELL cell;			/* used if command line assign to pseudo field */
     CELL *fp = (CELL *) 0;	/* ditto */
-    unsigned length;
+    size_t length;
 
     if (scan_code[*(unsigned char *) s] != SC_IDCHAR)
 	return 0;
@@ -583,3 +606,15 @@ is_cmdline_assign(char *s)
     }
     return 1;
 }
+
+#ifdef NO_LEAKS
+void
+fin_leaks(void)
+{
+    TRACE(("fin_leaks\n"));
+    if (main_fin) {
+	free_fin_data(main_fin);
+	main_fin = 0;
+    }
+}
+#endif
