@@ -1,6 +1,7 @@
 /********************************************
 hash.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2009,2010, Thomas E. Dickey
+copyright 1991-1993,1994, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: hash.c,v 1.6 2009/09/20 22:24:06 tom Exp $
+ * $MawkId: hash.c,v 1.18 2010/12/10 17:00:00 tom Exp $
  * @Log: hash.c,v @
  * Revision 1.3  1994/10/08  19:15:43  mike
  * remove SM_DOS
@@ -32,7 +33,11 @@ the GNU General Public License, version 2, 1991.
 #include "memory.h"
 #include "symtype.h"
 
-/*                                                                              
+#ifdef NO_LEAKS
+#include "bi_vars.h"
+#endif
+
+/*
  * FNV-1 hash function
  * http://www.isthe.com/chongo/tech/comp/fnv/index.html
  */
@@ -50,7 +55,7 @@ hash(const char *s)
 }
 
 unsigned
-hash2(const char *s, unsigned len)
+hash2(const char *s, size_t len)
 {
     /* FNV-1 */
     register unsigned h = 2166136261U;
@@ -67,10 +72,14 @@ typedef struct hash {
     SYMTAB symtab;
 } HASHNODE;
 
-static HASHNODE *delete(const char *);
-
 static HASHNODE *hash_table[HASH_PRIME];
 
+#ifdef NO_LEAKS
+static void free_hashnode(HASHNODE *);
+#else
+#define free_hashnode(p) zfree(delete(p->symtab.name), sizeof(HASHNODE))
+#endif
+
 /*
 insert a string in the symbol table.
 Caller knows the symbol is not there
@@ -85,6 +94,9 @@ insert(const char *s)
 
     p->link = hash_table[h = hash(s) % HASH_PRIME];
     p->symtab.name = s;
+#ifdef NO_LEAKS
+    p->symtab.free_name = 0;
+#endif
     hash_table[h] = p;
     return &p->symtab;
 }
@@ -106,6 +118,9 @@ find(const char *s)
 	    p = ZMALLOC(HASHNODE);
 	    p->symtab.type = ST_NONE;
 	    p->symtab.name = strcpy(zmalloc(strlen(s) + 1), s);
+#ifdef NO_LEAKS
+	    p->symtab.free_name = 1;
+#endif
 	    break;
 	}
 
@@ -189,7 +204,7 @@ save_id(const char *s)
     return &q->symtab;
 }
 
-/* restore all global indentifiers */
+/* restore all global identifiers */
 void
 restore_ids(void)
 {
@@ -201,7 +216,7 @@ restore_ids(void)
     while (q) {
 	p = q;
 	q = q->link;
-	zfree(delete(p->symtab.name), sizeof(HASHNODE));
+	free_hashnode(p);
 	p->link = hash_table[h = last_hash];
 	hash_table[h] = p;
     }
@@ -258,3 +273,67 @@ reverse_find(int type, PTR ptr)
     }
     return uk;
 }
+
+#ifdef NO_LEAKS
+static void
+free_symtab_name(HASHNODE * p)
+{
+    if (p->symtab.free_name) {
+	zfree((PTR) (p->symtab.name), strlen(p->symtab.name) + 1);
+    }
+}
+
+static void
+free_hashnode(HASHNODE * p)
+{
+    CELL *cp;
+
+    TRACE(("...deleting hash %p (%p) %s %d\n", p, &(p->symtab),
+	   p->symtab.name, p->symtab.type));
+    p = delete(p->symtab.name);
+    switch (p->symtab.type) {
+    case ST_FUNCT:
+	free_codes(p->symtab.name,
+		   p->symtab.stval.fbp->code,
+		   p->symtab.stval.fbp->size);
+	if (p->symtab.stval.fbp->nargs)
+	    zfree(p->symtab.stval.fbp->typev, p->symtab.stval.fbp->nargs);
+	zfree(p->symtab.stval.fbp, sizeof(FBLOCK));
+	break;
+    case ST_NONE:
+	break;
+    case ST_VAR:
+	cp = p->symtab.stval.cp;
+	if (cp != 0
+	    && (cp < bi_vars || cp > bi_vars + NUM_BI_VAR)) {
+	    switch (cp->type) {
+	    case C_STRING:
+	    case C_STRNUM:
+	    case C_MBSTRN:
+		free_STRING(string(cp));
+		break;
+	    }
+	    zfree(cp, sizeof(CELL));
+	}
+	break;
+    default:
+	break;
+    }
+    free_symtab_name(p);
+    zfree(p, sizeof(HASHNODE));
+}
+
+void
+hash_leaks(void)
+{
+    int i;
+    HASHNODE *p;
+
+    TRACE(("hash_leaks\n"));
+    for (i = 0; i < HASH_PRIME; i++) {
+	while ((p = hash_table[i]) != 0) {
+	    free_hashnode(p);
+	}
+    }
+}
+#endif
