@@ -1,6 +1,7 @@
 /********************************************
 init.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2010,2012, Thomas E. Dickey
+copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.c,v 1.16 2009/12/16 23:46:43 tom Exp $
+ * $MawkId: init.c,v 1.27 2012/06/27 17:57:14 tom Exp $
  * @Log: init.c,v @
  * Revision 1.11  1995/08/20  17:35:21  mike
  * include <stdlib.h> for MSC, needed for environ decl
@@ -75,10 +76,20 @@ the GNU General Public License, version 2, 1991.
 
 #ifdef MSDOS
 #include <fcntl.h>
-#ifdef MSDOS_MSC
-#include <stdlib.h>
 #endif
+
+typedef enum {
+    W_UNKNOWN = 0,
+#if USE_BINMODE
+    W_BINMODE,
 #endif
+    W_VERSION,
+    W_DUMP,
+    W_INTERACTIVE,
+    W_EXEC,
+    W_SPRINTF,
+    W_POSIX_SPACE
+} W_OPTIONS;
 
 static void process_cmdline(int, char **);
 static void set_ARGV(int, char **, int);
@@ -124,7 +135,7 @@ initialize(int argc, char **argv)
 
     code_init();
     fpe_init();
-    set_stderr();
+    set_stdio();
 
 #if USE_BINMODE
     stdout_init();
@@ -151,14 +162,111 @@ no_program(void)
     mawk_exit(0);
 }
 
+/*
+ * Compare ignoring case, but warn about mismatches.
+ */
+static int
+ok_abbrev(const char *fullName, const char *partName, int partLen)
+{
+    int result = 1;
+    int n;
+
+    for (n = 0; n < partLen; ++n) {
+	UChar ch = (UChar) partName[n];
+	if (isalpha(ch))
+	    ch = (UChar) toupper(ch);
+	if (ch != (UChar) fullName[n]) {
+	    result = 0;
+	    break;
+	}
+    }
+    return result;
+}
+
+static char *
+skipValue(char *value)
+{
+    while (*value != '\0' && *value != ',') {
+	++value;
+    }
+    return value;
+}
+
+static int
+haveValue(char *value)
+{
+    int result = 0;
+
+    if (*value++ == '=') {
+	if (*value != '\0' && strchr("=,", *value) == 0)
+	    result = 1;
+    }
+    return result;
+}
+
+static W_OPTIONS
+parse_w_opt(char *source, char **next)
+{
+#define DATA(name) { W_##name, #name }
+    static const struct {
+	W_OPTIONS code;
+	const char *name;
+    } w_options[] = {
+	DATA(VERSION),
+#if USE_BINMODE
+	    DATA(BINMODE),
+#endif
+	    DATA(DUMP),
+	    DATA(INTERACTIVE),
+	    DATA(EXEC),
+	    DATA(SPRINTF),
+	    DATA(POSIX_SPACE)
+    };
+#undef DATA
+    W_OPTIONS result = W_UNKNOWN;
+    int n;
+    int match = -1;
+    const char *first;
+
+    /* forgive and ignore empty options */
+    while (*source != '\0' && *source == ',') {
+	++source;
+    }
+
+    first = source;
+    if (*source != '\0') {
+	while (*source != '\0' && *source != ',' && *source != '=') {
+	    ++source;
+	}
+	for (n = 0; n < (int) (sizeof(w_options) / sizeof(w_options[0])); ++n) {
+	    if (ok_abbrev(w_options[n].name, first, (int) (source - first))) {
+		if (match >= 0) {
+		    errmsg(0, "? ambiguous -W value: %s vs %s\n",
+			   w_options[match].name,
+			   w_options[n].name);
+		} else {
+		    match = n;
+		}
+	    }
+	}
+    }
+    *next = source;
+
+    if (match >= 0)
+	result = w_options[match].code;
+
+    return result;
+}
+
 static void
 process_cmdline(int argc, char **argv)
 {
-    int i, nextarg;
+    int i, j, nextarg;
     char *optArg;
+    char *optNext;
     PFILE dummy;		/* starts linked list of filenames */
     PFILE *tail = &dummy;
-    unsigned length;
+    size_t length;
 
     for (i = 1; i < argc && argv[i][0] == '-'; i = nextarg) {
 	if (argv[i][1] == 0)	/* -  alone */
@@ -172,7 +280,7 @@ process_cmdline(int argc, char **argv)
 	/*
 	 * Check for "long" options and decide how to handle them.
 	 */
-	if (strlen(argv[i]) > 2 && !strncmp(argv[i], "--", 2)) {
+	if (strlen(argv[i]) > 2 && !strncmp(argv[i], "--", (size_t) 2)) {
 	    char *env = getenv("MAWK_LONG_OPTIONS");
 	    if (env != 0) {
 		switch (*env) {
@@ -210,49 +318,72 @@ process_cmdline(int argc, char **argv)
 	}
 
 	switch (argv[i][1]) {
+
 	case 'W':
+	    for (j = 0; j < (int) strlen(optArg); j = (int) (optNext - optArg)) {
+		switch (parse_w_opt(optArg + j, &optNext)) {
+		case W_VERSION:
+		    print_version();
+		    break;
+#if USE_BINMODE
+		case W_BINMODE:
+		    if (haveValue(optNext)) {
+			set_binmode(atoi(optNext + 1));
+			optNext = skipValue(optNext);
+		    } else {
+			errmsg(0, "missing value for -W binmode");
+			mawk_exit(2);
+		    }
+		    break;
+#endif
+		case W_DUMP:
+		    dump_code_flag = 1;
+		    break;
 
-	    if (isalpha((UChar) optArg[0]))
-		optArg[0] = (char) toupper((UChar) optArg[0]);
-	    if (optArg[0] == 'V')
-		print_version();
-	    else if (optArg[0] == 'D') {
-		dump_code_flag = 1;
-	    } else if (optArg[0] == 'S') {
-		char *p = strchr(optArg, '=');
-		int x = p ? atoi(p + 1) : 0;
+		case W_EXEC:
+		    if (pfile_name) {
+			errmsg(0, "-W exec is incompatible with -f");
+			mawk_exit(2);
+		    } else if (nextarg == argc) {
+			no_program();
+		    }
+		    pfile_name = argv[nextarg];
+		    i = nextarg + 1;
+		    goto no_more_opts;
 
-		if (x > (int) SPRINTF_SZ) {
-		    sprintf_buff = (char *) zmalloc((unsigned) x);
-		    sprintf_limit = sprintf_buff + x;
+		case W_INTERACTIVE:
+		    interactive_flag = 1;
+		    setbuf(stdout, (char *) 0);
+		    break;
+
+		case W_POSIX_SPACE:
+		    posix_space_flag = 1;
+		    break;
+
+		case W_SPRINTF:
+		    if (haveValue(optNext)) {
+			int x = atoi(optNext + 1);
+
+			if (x > (int) SPRINTF_SZ) {
+			    sprintf_buff = (char *) zmalloc((size_t) x);
+			    sprintf_limit = sprintf_buff + x;
+			}
+			optNext = skipValue(optNext);
+		    } else {
+			errmsg(0, "missing value for -W sprintf");
+			mawk_exit(2);
+		    }
+		    break;
+
+		case W_UNKNOWN:
+		    errmsg(0, "vacuous option: -W %s", optArg + j);
+		    break;
+		}
+		while (*optNext == '=') {
+		    errmsg(0, "unexpected option value %s", optArg + j);
+		    optNext = skipValue(optNext);
 		}
 	    }
-#if USE_BINMODE
-	    else if (optArg[0] == 'B') {
-		char *p = strchr(optArg, '=');
-		int x = p ? atoi(p + 1) : 0;
-
-		set_binmode(x);
-	    }
-#endif
-	    else if (optArg[0] == 'P') {
-		posix_space_flag = 1;
-	    } else if (optArg[0] == 'E') {
-		if (pfile_name) {
-		    errmsg(0, "-W exec is incompatible with -f");
-		    mawk_exit(2);
-		} else if (nextarg == argc)
-		    no_program();
-
-		pfile_name = argv[nextarg];
-		i = nextarg + 1;
-		goto no_more_opts;
-	    } else if (optArg[0] == 'I') {
-		interactive_flag = 1;
-		setbuf(stdout, (char *) 0);
-	    } else
-		errmsg(0, "vacuous option: -W %s", optArg);
-
 	    break;
 
 	case 'v':
@@ -331,6 +462,7 @@ set_ARGV(int argc, char **argv, int i)
     st_p = insert("ARGV");
     st_p->type = ST_ARRAY;
     Argv = st_p->stval.array = new_ARRAY();
+    no_leaks_array(Argv);
     argi.type = C_DOUBLE;
     argi.dval = 0.0;
     cp = array_find(st_p->stval.array, &argi, CREATE);
@@ -372,7 +504,7 @@ load_environ(ARRAY ENV)
 
     while (*p) {
 	if ((s = strchr(*p, '='))) {	/* shouldn't fail */
-	    unsigned len = (unsigned) (s - *p);
+	    size_t len = (size_t) (s - *p);
 	    c.ptr = (PTR) new_STRING0(len);
 	    memcpy(string(&c)->str, *p, len);
 	    s++;
@@ -386,3 +518,34 @@ load_environ(ARRAY ENV)
 	p++;
     }
 }
+
+#ifdef NO_LEAKS
+typedef struct _all_arrays {
+    struct _all_arrays *next;
+    ARRAY a;
+} ALL_ARRAYS;
+
+static ALL_ARRAYS *all_arrays;
+
+void
+array_leaks(void)
+{
+    while (all_arrays != 0) {
+	ALL_ARRAYS *next = all_arrays->next;
+	array_clear(all_arrays->a);
+	ZFREE(all_arrays->a);
+	free(all_arrays);
+	all_arrays = next;
+    }
+}
+
+/* use this to identify array leaks */
+void
+no_leaks_array(ARRAY a)
+{
+    ALL_ARRAYS *p = calloc(1, sizeof(ALL_ARRAYS));
+    p->next = all_arrays;
+    p->a = a;
+    all_arrays = p;
+}
+#endif
