@@ -1,6 +1,7 @@
 /********************************************
 main.c
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 1991-1993,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: main.c,v 1.8 2009/12/14 00:23:56 tom Exp $
+ * $MawkId: main.c,v 1.23 2010/12/10 17:00:00 tom Exp $
  * @Log: main.c,v @
  * Revision 1.4  1995/06/09  22:57:19  mike
  * parse() no longer returns on error
@@ -44,6 +45,7 @@ the GNU General Public License, version 2, 1991.
 /*  main.c  */
 
 #include "mawk.h"
+#include "bi_vars.h"
 #include "init.h"
 #include "code.h"
 #include "files.h"
@@ -87,6 +89,24 @@ mawk_exit(int x)
 #ifdef  HAVE_FAKE_PIPES
     close_fake_pipes();
 #endif
+#endif
+
+#ifdef NO_LEAKS
+    code_leaks();
+    scan_leaks();
+    cell_leaks();
+    re_leaks();
+    rexp_leaks();
+    bi_vars_leaks();
+    hash_leaks();
+    array_leaks();
+    files_leaks();
+    fin_leaks();
+    field_leaks();
+    zmalloc_leaks();
+#if OPT_TRACE > 0
+    trace_leaks();
+#endif
 #endif
 
     exit(x);
