@@ -1,6 +1,7 @@
 /********************************************
 mawk.h
-copyright 1991-94, Michael D. Brennan
+copyright 2008-2009,2010 Thomas E. Dickey
+copyright 1991-1995,1996 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: mawk.h,v 1.16 2009/12/16 23:29:20 tom Exp $
+ * $MawkId: mawk.h,v 1.37 2010/12/10 17:00:00 tom Exp $
  * @Log: mawk.h,v @
  * Revision 1.10  1996/08/25 19:31:04  mike
  * Added work-around for solaris strtod overflow bug.
@@ -41,8 +42,7 @@ the GNU General Public License, version 2, 1991.
  *
  * Revision 1.2  1993/07/04  12:52:06  mike
  * start on autoconfig changes
- *
-*/
+ */
 
 /*  mawk.h  */
 
@@ -50,21 +50,36 @@ the GNU General Public License, version 2, 1991.
 #define  MAWK_H
 
 #include "nstd.h"
+
 #include <stdio.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
+#include <assert.h>
+
 #include "types.h"
 
 #ifndef GCC_NORETURN
 #define GCC_NORETURN		/* nothing */
 #endif
 
+#ifndef GCC_PRINTFLIKE
+#define  GCC_PRINTFLIKE(fmt,var)	/* nothing */
+#endif
+
+#ifndef GCC_UNUSED
+#define GCC_UNUSED		/* nothing */
+#endif
+
 #ifdef   DEBUG
 #define  YYDEBUG  1
 extern int yydebug;		/* print parse if on */
 extern int dump_RE;
 #endif
 
-#if defined(MSDOS) || defined(__MINGW32__)
+#if defined(MSDOS) || defined(__MINGW32__) || defined(_WINNT)
 #define USE_BINMODE 1
 #else
 #define USE_BINMODE 0
@@ -126,64 +141,123 @@ extern unsigned rt_nr, rt_fnr;	/* ditto */
 #ifdef   DEBUG
 #define cell_destroy(cp)  DB_cell_destroy(cp)
 #else
-
+/* Note: type is only C_STRING to C_MBSTRN */
 #define cell_destroy(cp) \
 	do { \
 	    if ( (cp)->type >= C_STRING && \
-	       -- string(cp)->ref_cnt == 0 ) \
-	    zfree(string(cp),string(cp)->len+STRING_OH); \
+	         (cp)->type <= C_MBSTRN ) { \
+		free_STRING(string(cp));  \
+	    } \
 	} while (0)
 #endif
 
 /*  prototypes  */
 
-void cast1_to_s(CELL *);
-void cast1_to_d(CELL *);
-void cast2_to_s(CELL *);
-void cast2_to_d(CELL *);
-void cast_to_RE(CELL *);
-void cast_for_split(CELL *);
-void check_strnum(CELL *);
-void cast_to_REPL(CELL *);
-Int d_to_I(double);
-UInt d_to_U(double d);
+extern void cast1_to_s(CELL *);
+extern void cast1_to_d(CELL *);
+extern void cast2_to_s(CELL *);
+extern void cast2_to_d(CELL *);
+extern void cast_to_RE(CELL *);
+extern void cast_for_split(CELL *);
+extern void check_strnum(CELL *);
+extern void cast_to_REPL(CELL *);
+extern Int d_to_I(double);
+extern UInt d_to_U(double d);
 
 #define d_to_i(d)     ((int)d_to_I(d))
 
-int test(CELL *);		/* test for null non-null */
-CELL *cellcpy(CELL *, CELL *);
-CELL *repl_cpy(CELL *, CELL *);
-void DB_cell_destroy(CELL *);
-void overflow(const char *, unsigned);
-void rt_overflow(const char *, unsigned);
-void rt_error(const char *,...) GCC_NORETURN;
-void mawk_exit(int) GCC_NORETURN;
-void da(INST *, FILE *);
-char *rm_escape(char *, unsigned *);
-char *re_pos_match(char *, size_t, PTR, unsigned *);
-int binmode(void);
+extern int test(CELL *);	/* test for null non-null */
+extern CELL *cellcpy(CELL *, CELL *);
+extern CELL *repl_cpy(CELL *, CELL *);
+extern void DB_cell_destroy(CELL *);
+extern void overflow(const char *, unsigned);
+extern void rt_overflow(const char *, unsigned);
+extern void rt_error(const char *,...) GCC_NORETURN  GCC_PRINTFLIKE(1,2);
+extern void mawk_exit(int) GCC_NORETURN;
+extern void da(INST *, FILE *);
+extern char *rm_escape(char *, size_t *);
+extern char *re_pos_match(char *, size_t, PTR, size_t *);
+extern int binmode(void);
 
 #ifndef  REXP_H
-char *str_str(char *, unsigned, char *, unsigned);
+extern char *str_str(char *, size_t, char *, size_t);
 #endif
 
-void parse(void);
-int yylex(void);
-int yyparse(void);
-void yyerror(char *);
-void scan_cleanup(void);
+extern void parse(void);
+extern int yylex(void);
+extern int yyparse(void);
+extern void yyerror(const char *);
+extern void scan_cleanup(void);
 
-void bozo(const char *) GCC_NORETURN;
-void errmsg(int, const char *,...);
-void compile_error(const char *,...);
+extern void bozo(const char *) GCC_NORETURN;
+extern void errmsg(int, const char *,...) GCC_PRINTFLIKE(2,3);
+extern void compile_error(const char *,...) GCC_PRINTFLIKE(1,2);
 
-void execute(INST *, CELL *, CELL *);
-const char *find_kw_str(int);
-void da_string(FILE *fp, const char *, unsigned);
+extern void execute(INST *, CELL *, CELL *);
+extern const char *find_kw_str(int);
+extern void da_string(FILE *fp, const char *, size_t);
 
 #ifdef HAVE_STRTOD_OVF_BUG
-double strtod_with_ovf_bug(const char *, char **);
+extern double strtod_with_ovf_bug(const char *, char **);
 #define strtod  strtod_with_ovf_bug
 #endif
 
+#if OPT_TRACE > 0
+extern void Trace(const char *,...) GCC_PRINTFLIKE(1,2);
+#define TRACE(params) Trace params
+#if OPT_TRACE > 1
+#define TRACE2(params) Trace params
+#endif
+#endif
+
+#ifndef TRACE
+#define TRACE(params)		/* nothing */
+#endif
+
+#ifndef TRACE2
+#define TRACE2(params)		/* nothing */
+#endif
+
+#ifdef NO_LEAKS
+
+extern const char *da_op_name(INST *);
+extern void free_cell_data(CELL *);
+extern void free_codes(const char *, INST *, size_t);
+extern void no_leaks_cell(CELL *);
+extern void no_leaks_cell_ptr(CELL *);
+extern void no_leaks_re_ptr(PTR);
+
+extern void array_leaks(void);
+extern void bi_vars_leaks(void);
+extern void cell_leaks(void);
+extern void code_leaks(void);
+extern void field_leaks(void);
+extern void files_leaks(void);
+extern void fin_leaks(void);
+extern void hash_leaks(void);
+extern void re_leaks(void);
+extern void rexp_leaks(void);
+extern void scan_leaks(void);
+extern void trace_leaks(void);
+extern void zmalloc_leaks(void);
+
+#else
+
+#define free_codes(tag, base, size) zfree(base, size)
+#define no_leaks_cell(ptr)	/* nothing */
+#define no_leaks_cell_ptr(ptr)	/* nothing */
+#define no_leaks_re_ptr(ptr)	/* nothing */
+
+#endif
+
+/*
+ * Sometimes split_buff[] pointers are moved rather than copied.
+ * Optimize-out the assignment to clear the pointer in the array.
+ */
+#ifdef NO_LEAKS
+#define USED_SPLIT_BUFF(n) split_buff[n] = 0
+#else
+#define USED_SPLIT_BUFF(n)	/* nothing */
+#endif
+
 #endif /* MAWK_H */
