@@ -1,6 +1,7 @@
 /********************************************
 print.c
-copyright 1991-1993.  Michael D. Brennan
+copyright 2008-2009,2010.  Thomas E. Dickey
+copyright 1991-1995,1996.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: print.c,v 1.19 2009/12/14 01:10:05 tom Exp $
+ * $MawkId: print.c,v 1.22 2010/12/10 17:00:00 tom Exp $
  * @Log: print.c,v @
  * Revision 1.7  1996/09/18 01:04:36  mike
  * Check ferror() after print and printf.
@@ -84,7 +85,7 @@ char *sprintf_limit = string_buff + SPRINTF_SZ;
 static void
 print_cell(CELL * p, FILE *fp)
 {
-    unsigned len;
+    size_t len;
 
     switch (p->type) {
     case C_NOINIT:
@@ -100,7 +101,7 @@ print_cell(CELL * p, FILE *fp)
 	    break;
 
 	default:
-	    fwrite(string(p)->str, 1, len, fp);
+	    fwrite(string(p)->str, (size_t) 1, len, fp);
 	}
 	break;
 
@@ -341,7 +342,7 @@ static char *
 SprintfFill(char *buffer, int ch, int fill)
 {
     SprintfOverflow(buffer, fill);
-    memset(buffer, ch, (unsigned) fill);
+    memset(buffer, ch, (size_t) fill);
     return buffer + fill;
 }
 
@@ -349,7 +350,7 @@ static char *
 SprintfBlock(char *buffer, char *source, int length)
 {
     SprintfOverflow(buffer, length);
-    memcpy(buffer, source, (unsigned) length);
+    memcpy(buffer, source, (size_t) length);
     return buffer + length;
 }
 
@@ -392,7 +393,7 @@ puts_sfmt(PTR target,
 		--width;
 	    }
 	}
-	fwrite(src_str, sizeof(char), (unsigned) src_len, fp);
+	fwrite(src_str, sizeof(char), (size_t) src_len, fp);
 	while (src_len < width) {
 	    fputc(' ', fp);
 	    --width;
@@ -427,7 +428,7 @@ puts_sfmt(PTR target,
     return target;
 }
 
-/* 
+/*
  * Note: caller must do CELL cleanup.
  * The format parameter is modified, but restored.
  *
@@ -494,7 +495,7 @@ do_printf(
 				    (unsigned) (sprintf_limit - sprintf_buff));
 		    } else {	/* really done */
 			return new_STRING1(sprintf_buff,
-					   (unsigned) (target - sprintf_buff));
+					   (size_t) (target - sprintf_buff));
 		    }
 		} else {
 		    *target++ = *q++;
