@@ -1,6 +1,8 @@
 /********************************************
 scan.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2009,2010, Thomas E. Dickey
+copyright 2010, Jonathan Nieder
+copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: scan.c,v 1.14 2009/12/16 09:57:25 tom Exp $
+ * $MawkId: scan.c,v 1.30 2010/12/10 17:00:00 tom Exp $
  * @Log: scan.c,v @
  * Revision 1.8  1996/07/28 21:47:05  mike
  * gnuish patch
@@ -76,6 +78,9 @@ the GNU General Public License, version 2, 1991.
 
 #include  "files.h"
 
+double double_zero = 0.0;
+double double_one = 1.0;
+
 /* static functions */
 static void scan_fillbuff(void);
 static void scan_open(void);
@@ -112,7 +117,7 @@ scan_init(char *cmdline_program)
 	eof_flag = 1;
     } else {			/* program from file[s] */
 	scan_open();
-	buffp = buffer = (UChar *) zmalloc(BUFFSZ + 1);
+	buffp = buffer = (UChar *) zmalloc((size_t) (BUFFSZ + 1));
 	scan_fillbuff();
     }
 
@@ -145,8 +150,8 @@ void
 scan_cleanup(void)
 {
     if (program_fd >= 0)
-	zfree(buffer, BUFFSZ + 1);
-    else
+	zfree(buffer, (size_t) (BUFFSZ + 1));
+    if (program_string)
 	free_STRING(program_string);
 
     if (program_fd > 0)
@@ -187,9 +192,9 @@ static unsigned lineno = 1;
 static void
 scan_fillbuff(void)
 {
-    unsigned r;
+    size_t r;
 
-    r = fillbuff(program_fd, (char *) buffer, BUFFSZ);
+    r = fillbuff(program_fd, (char *) buffer, (size_t) BUFFSZ);
     if (r < BUFFSZ) {
 	eof_flag = 1;
 	/* make sure eof is terminated */
@@ -314,6 +319,10 @@ yylex(void)
 
     token_lineno = lineno;
 
+#ifdef NO_LEAKS
+    memset(&yylval, 0, sizeof(yylval));
+#endif
+
   reswitch:
 
     switch (scan_code[c = next()]) {
@@ -448,7 +457,7 @@ yylex(void)
 	ct_ret(RBOX);
 
     case SC_MATCH:
-	string_buff[0] = '~';
+	string_buff[1] = '~';
 	string_buff[0] = 0;
 	yylval.ival = 1;
 	ct_ret(MATCH);
@@ -568,8 +577,6 @@ yylex(void)
 	{
 	    double d;
 	    int flag;
-	    static double double_zero = 0.0;
-	    static double double_one = 1.0;
 
 	    if ((d = collect_decimal(c, &flag)) == 0.0) {
 		if (flag)
@@ -644,6 +651,7 @@ yylex(void)
 			zmalloc(sizeof(FBLOCK));
 		    stp->stval.fbp->name = stp->name;
 		    stp->stval.fbp->code = (INST *) 0;
+		    stp->stval.fbp->size = 0;
 		    yylval.fbp = stp->stval.fbp;
 		    current_token = FUNCT_ID;
 		} else {
@@ -896,7 +904,7 @@ escape_test[ET_END + 1] =
 };
 /* *INDENT-ON* */
 /* process the escape characters in a string, in place . */ char *
-rm_escape(char *s, unsigned *lenp)
+rm_escape(char *s, size_t *lenp)
 {
     register char *p, *q;
     char *t;
@@ -945,7 +953,7 @@ collect_string(void)
     register UChar *p = (UChar *) string_buff;
     int c;
     int e_flag = 0;		/* on if have an escape char */
-    unsigned len_buff;
+    size_t len_buff;
 
     while (1)
 	switch (scan_code[*p++ = (UChar) next()]) {
@@ -960,7 +968,7 @@ collect_string(void)
 	case 0:		/* unterminated string */
 	    compile_error(
 			     "runaway string constant \"%.10s ...",
-			     string_buff, token_lineno);
+			     string_buff);
 	    mawk_exit(2);
 
 	case SC_ESCAPE:
@@ -993,23 +1001,26 @@ static int
 collect_RE(void)
 {
     char *p = string_buff;
-    char *first = string_buff;
+    const char *first = NULL;
+    int limit = MIN_SPRINTF - 2;
     int c;
     int boxed = 0;
     STRING *sval;
 
     while (1) {
-	if (p == (string_buff + MIN_SPRINTF - 2)) {
+	if (p >= (string_buff + limit)) {
 	    compile_error(
 			     "regular expression /%.10s ..."
-			     " exceeds implementation size limit",
-			     string_buff);
+			     " exceeds implementation size limit (%d)",
+			     string_buff,
+			     limit);
 	    mawk_exit(2);
 	}
 	c = (UChar) (*p++ = (char) next());
 	switch (scan_code[c]) {
 	case SC_POW:
-	    if (p == first + 1) {
+	    /* Handle [^]] and [^^] correctly. */
+	    if ((p - 1) == first && first != 0 && first[-1] == '[') {
 		first = p;
 	    }
 	    break;
@@ -1023,9 +1034,15 @@ collect_RE(void)
 	    if (!boxed) {
 		first = p;
 		++boxed;
-	    } else if (p != first + 1) {
-		++boxed;
 	    } else {
+		/* XXX. Does not handle collating symbols or equivalence
+		 * class expressions. */
+		/* XXX. Does not match logic used in rexp0.c to check for
+		 * a character class expression, though probably the
+		 * latter should be adjusted.
+		 * POSIX and common sense give us license to complain about
+		 * expressions such as '[[:not a special character class]]'.
+		 */
 		if (next() == ':') {
 		    ++boxed;
 		}
@@ -1038,7 +1055,7 @@ collect_RE(void)
 	     * A right square-bracket loses its special meaning if it occurs
 	     * first in the list (after an optional "^").
 	     */
-	    if (p != first + 1) {
+	    if (boxed && p - 1 != first) {
 		--boxed;
 	    }
 	    break;
@@ -1057,7 +1074,7 @@ collect_RE(void)
 	case 0:		/* unterminated re */
 	    compile_error(
 			     "runaway regular expression /%.10s ...",
-			     string_buff, token_lineno);
+			     string_buff);
 	    mawk_exit(2);
 
 	case SC_ESCAPE:
@@ -1089,3 +1106,15 @@ collect_RE(void)
     free_STRING(sval);
     return RE;
 }
+
+#ifdef NO_LEAKS
+void
+scan_leaks(void)
+{
+    TRACE(("scan_leaks\n"));
+    if (yylval.ptr) {
+	free(yylval.ptr);
+	yylval.ptr = 0;
+    }
+}
+#endif
