@@ -1,6 +1,7 @@
 /********************************************
 version.c
-copyright 1991-95.  Michael D. Brennan
+copyright 2008-2009,2010.  Thomas E. Dickey
+copyright 1991-1995,1996.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,8 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: version.c,v 1.7 2009/12/16 00:58:04 tom Exp $
+ * $MawkId: version.c,v 1.11 2010/12/10 17:00:00 tom Exp $
+ *
  * @Log: version.c,v @
  * Revision 1.10  1996/07/28 21:47:07  mike
  * gnuish patch
@@ -31,8 +33,8 @@ the GNU General Public License, version 2, 1991.
 #include "patchlev.h"
 
 #define	 VERSION_STRING	 \
-  "mawk 1.3%s%s %s\n\
-Copyright 2009, Thomas E. Dickey\n\
+  "mawk %d.%d%s%s %s\n\
+Copyright 2010, Thomas E. Dickey\n\
 Copyright 1996, Michael D. Brennan\n\n"
 
 /* If use different command line syntax for MSDOS
@@ -113,7 +115,8 @@ print_aux_limits(void)
     fprintf(stderr, fmt, "heap size", (unsigned long) coreleft());
 #endif
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && (_MSC_VER < 800)
+    _MSC_VER;
     fprintf(stderr, fmt, "stack size", (unsigned long) stackavail());
 #endif
 
@@ -130,8 +133,9 @@ print_aux_limits(void)
 void
 print_version(void)
 {
-
-    printf(VERSION_STRING, PATCH_STRING, DOS_STRING, DATE_STRING);
+    printf(VERSION_STRING,
+	   PATCH_BASE, PATCH_LEVEL, PATCH_STRING,
+	   DOS_STRING, DATE_STRING);
     fflush(stdout);
 
 #ifdef LOCAL_REGEXP
@@ -144,5 +148,5 @@ print_version(void)
     fprintf(stderr, fmt, "max NF", (long) MAX_FIELD);
     fprintf(stderr, fmt, "sprintf buffer", (long) SPRINTF_SZ);
     print_aux_limits();
-    exit(0);
+    mawk_exit(0);
 }
